"""Auto-generated file, do not edit by hand. RU metadata"""
from phonenumbers.phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_RU = PhoneMetadata(id='RU', country_code=7, international_prefix='810',
    general_desc=PhoneNumberDesc(national_number_pattern='[347-9]\\d{9}', possible_length=(10,)),
    fixed_line=PhoneNumberDesc(national_number_pattern='[348]\\d{9}', example_number='3011234567', possible_length=(10,)),
    mobile=PhoneNumberDesc(national_number_pattern='9\\d{9}', example_number='9123456789', possible_length=(10,)),
    national_prefix='8',
    national_prefix_for_parsing='8')



"""Auto-generated file, do not edit by hand. CN metadata"""
from phonenumbers.phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_CN = PhoneMetadata(id='CN', country_code=86, international_prefix='00',
    general_desc=PhoneNumberDesc(national_number_pattern='[1-7]\\d{6,11}|8[0-357-9]\\d{6,9}|9\\d{7,10}', possible_length=(11,)),
    fixed_line=PhoneNumberDesc(national_number_pattern='[2-9]\\d{10}', example_number='91234567', possible_length=(11,)),
    mobile=PhoneNumberDesc(national_number_pattern='1(?:[38]\\d|4[57]|5[0-35-9]|7[0136-8])\\d{8}', example_number='13123456789', possible_length=(11,)),
    national_prefix='0',
    national_prefix_for_parsing='0',
    number_format=[NumberFormat(pattern='(\\d{3})(\\d{5,6})', format='\\1 \\2', leading_digits_pattern=['[3-9]', '[3-9]\\d{2}[19]', '[3-9]\\d{2}(?:10|95)'], national_prefix_formatting_rule='0\\1', domestic_carrier_code_formatting_rule='$CC \\1'),
        NumberFormat(pattern='(\\d{3})(\\d{8})', format='\\1 \\2', leading_digits_pattern=['1'], national_prefix_formatting_rule='\\1')])



"""Auto-generated file, do not edit by hand. NZ metadata"""
from phonenumbers.phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_NZ = PhoneMetadata(id='NZ', country_code=64, international_prefix='00',
    general_desc=PhoneNumberDesc(national_number_pattern='[289]\\d{7,9}|[3-7]\\d{7}', possible_length=(7, 8, 9, 10)),
    fixed_line=PhoneNumberDesc(national_number_pattern='24099\\d{3}|(?:3[2-79]|[479][2-689]|6[235-9])\\d{6}', example_number='24099123', possible_length=(7, 8)),
    mobile=PhoneNumberDesc(national_number_pattern='2(?:[027]\\d{7}|9\\d{6,7}|1(?:0\\d{5,7}|[12]\\d{5,6}|[3-9]\\d{5})|4[1-9]\\d{6}|8\\d{7,8})', example_number='201234567', possible_length=(8, 9, 10)),
    toll_free=PhoneNumberDesc(national_number_pattern='800\\d{6,7}', example_number='8001234567', possible_length=(9, 10)),
    premium_rate=PhoneNumberDesc(national_number_pattern='900\\d{6,7}', example_number='9001234567', possible_length=(9, 10)),
    national_prefix='0',
    national_prefix_for_parsing='0',
    number_format=[NumberFormat(pattern='(\\d)(\\d{3})(\\d{4})', format='\\1-\\2 \\3', leading_digits_pattern=['24|[34679]'], national_prefix_formatting_rule='0\\1'),
        NumberFormat(pattern='(\\d)(\\d{3})(\\d{3,5})', format='\\1-\\2 \\3', leading_digits_pattern=['2[179]'], national_prefix_formatting_rule='0\\1'),
        NumberFormat(pattern='(\\d{3})(\\d{3})(\\d{3,4})', format='\\1 \\2 \\3', leading_digits_pattern=['[89]'], national_prefix_formatting_rule='0\\1')])



"""Unit tests for carrier.py"""

# Based on original Java code:
#     java/carrier/test/com/google/i18n/phonenumbers/PhoneNumberToCarrierMapperTest.java
# Copyright (C) 2013 The Libphonenumber Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

from phonenumbers import FrozenPhoneNumber
from phonenumbers import carrier
from phonenumbers.carrier import name_for_number, name_for_valid_number, safe_display_name
from phonenumbers.util import u

# Allow override library carrier metadata with the test metadata.
REAL_CARRIER_DATA = carrier.CARRIER_DATA
REAL_CARRIER_LONGEST_PREFIX = carrier.CARRIER_LONGEST_PREFIX
from .testcarrierdata import CARRIER_DATA as TEST_CARRIER_DATA
from .testcarrierdata import CARRIER_LONGEST_PREFIX as TEST_CARRIER_LONGEST_PREFIX


def reinstate_real_carrierdata():
    """Reinstate real phone number carrier metadata"""
    carrier.CARRIER_DATA = REAL_CARRIER_DATA
    carrier.CARRIER_LONGEST_PREFIX = REAL_CARRIER_LONGEST_PREFIX


def insert_test_carrierdata():
    """Insert test carrier metadata into library"""
    carrier.CARRIER_DATA = TEST_CARRIER_DATA
    carrier.CARRIER_LONGEST_PREFIX = TEST_CARRIER_LONGEST_PREFIX


# Set up some test numbers to re-use.
AO_MOBILE1 = FrozenPhoneNumber(country_code=244, national_number=917654321)
AO_MOBILE2 = FrozenPhoneNumber(country_code=244, national_number=927654321)
AO_FIXED1 = FrozenPhoneNumber(country_code=244, national_number=22254321)
AO_FIXED2 = FrozenPhoneNumber(country_code=244, national_number=26254321)
AO_INVALID_NUMBER = FrozenPhoneNumber(country_code=244, national_number=101234)
UK_MOBILE1 = FrozenPhoneNumber(country_code=44, national_number=7387654321)
UK_MOBILE2 = FrozenPhoneNumber(country_code=44, national_number=7487654321)
UK_FIXED1 = FrozenPhoneNumber(country_code=44, national_number=1123456789)
UK_FIXED2 = FrozenPhoneNumber(country_code=44, national_number=2987654321)
UK_INVALID_NUMBER = FrozenPhoneNumber(country_code=44, national_number=7301234)
UK_PAGER = FrozenPhoneNumber(country_code=44, national_number=7601234567)
US_FIXED_OR_MOBILE = FrozenPhoneNumber(country_code=1, national_number=6502123456)
NUMBER_WITH_INVALID_COUNTRY_CODE = FrozenPhoneNumber(country_code=999, national_number=2423651234)
INTERNATIONAL_TOLL_FREE = FrozenPhoneNumber(country_code=800, national_number=12345678)

# Language/country codes
_ENGLISH = "en"
_FRENCH = "fr"


class PhoneNumberToCarrierMapperTest(unittest.TestCase):
    """Unit tests for carrier.py"""

    def setUp(self):
        insert_test_carrierdata()

    def tearDown(self):
        reinstate_real_carrierdata()

    def testGetDescriptionForMobilePortableRegion(self):
        self.assertEqual("British carrier", name_for_number(UK_MOBILE1, _ENGLISH))
        self.assertEqual(u("Brittisk operat\u00F6r"), name_for_number(UK_MOBILE1, "sv", region="SE"))
        self.assertEqual("British carrier", name_for_number(UK_MOBILE1, _FRENCH))
        # Returns an empty string because the UK implements mobile number portability.
        self.assertEqual("", safe_display_name(UK_MOBILE1, _ENGLISH))

    def testGetDescriptionForNonMobilePortableRegion(self):
        self.assertEqual("Angolan carrier", name_for_number(AO_MOBILE1, _ENGLISH))
        self.assertEqual("Angolan carrier", safe_display_name(AO_MOBILE1, _ENGLISH))

    def testGetDescriptionForFixedLineNumber(self):
        self.assertEqual("", name_for_number(AO_FIXED1, _ENGLISH))
        self.assertEqual("", name_for_number(UK_FIXED1, _ENGLISH))
        # If the carrier information is present in the files and the method
        # that assumes a valid number is used, a carrier is returned.
        self.assertEqual("Angolan fixed line carrier", name_for_valid_number(AO_FIXED2, _ENGLISH))
        self.assertEqual("", name_for_valid_number(UK_FIXED2, _ENGLISH))

    def testGetDescriptionForFixedOrMobileNumber(self):
        self.assertEqual("US carrier", name_for_number(US_FIXED_OR_MOBILE, _ENGLISH))

    def testGetDescriptionForPagerNumber(self):
        self.assertEqual("British pager", name_for_number(UK_PAGER, _ENGLISH))

    def testGetDescriptionForNumberWithNoDataFile(self):
        self.assertEqual("", name_for_number(NUMBER_WITH_INVALID_COUNTRY_CODE, _ENGLISH))
        self.assertEqual("", name_for_number(INTERNATIONAL_TOLL_FREE, _ENGLISH))
        self.assertEqual("", name_for_valid_number(NUMBER_WITH_INVALID_COUNTRY_CODE, _ENGLISH))
        self.assertEqual("", name_for_valid_number(INTERNATIONAL_TOLL_FREE, _ENGLISH))

    def testGetDescriptionForNumberWithMissingPrefix(self):
        self.assertEqual("", name_for_number(UK_MOBILE2, _ENGLISH))
        self.assertEqual("", name_for_number(AO_MOBILE2, _ENGLISH))

    def testGetDescriptionForInvalidNumber(self):
        self.assertEqual("", name_for_number(UK_INVALID_NUMBER, _ENGLISH))
        self.assertEqual("", name_for_number(AO_INVALID_NUMBER, _ENGLISH))



"""Auto-generated file, do not edit by hand. HU metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_HU = PhoneMetadata(id='HU', country_code=None, international_prefix=None,
    general_desc=PhoneNumberDesc(national_number_pattern='1\\d{2,5}', possible_length=(3, 4, 5, 6)),
    toll_free=PhoneNumberDesc(national_number_pattern='1(?:0[457]|1(?:2|6\\d{3}))', example_number='104', possible_length=(3, 6)),
    emergency=PhoneNumberDesc(national_number_pattern='1(?:0[457]|12)', example_number='104', possible_length=(3,)),
    short_code=PhoneNumberDesc(national_number_pattern='1(?:0[457]|1(?:2|6(?:000|1(?:11|23)))|2(?:20|7[02])|37(?:00|37|7[07])|414|777|8(?:1[27-9]|2[04]|40|[589]))', example_number='104', possible_length=(3, 4, 5, 6)),
    carrier_specific=PhoneNumberDesc(national_number_pattern='1(?:2[27]|41|77)\\d', example_number='1220', possible_length=(4,)),
    sms_services=PhoneNumberDesc(national_number_pattern='184\\d', example_number='1840', possible_length=(4,)),
    short_data=True)



"""Auto-generated file, do not edit by hand. SE metadata"""
from phonenumbers.phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_SE = PhoneMetadata(id='SE', country_code=46, international_prefix='00',
    general_desc=PhoneNumberDesc(national_number_pattern='\\d{9}', possible_length=(9,)),
    fixed_line=PhoneNumberDesc(example_number='123456789'))



from ..phonemetadata import NumberFormat

_AVAILABLE_REGION_CODES: list[str]
_AVAILABLE_NONGEO_COUNTRY_CODES: list[int]

def _load_region(code: str | int) -> None: ...

_ALT_NUMBER_FORMATS: dict[int, list[NumberFormat]]
_COUNTRY_CODE_TO_REGION_CODE: dict[int, tuple[str, ...]]



"""Per-prefix data, mapping each prefix to a dict of locale:name.

Auto-generated file, do not edit by hand.
"""
from phonenumbers.util import u

# Copyright (C) 2011-2021 The Libphonenumber Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

data = {
 '1650212':{'en': 'US carrier'},
 '1650213':{'en': 'US carrier2'},
 '244262':{'en': 'Angolan fixed line carrier'},
 '244917':{'en': 'Angolan carrier'},
 '4411':{'en': 'British fixed line carrier'},
 '4473':{'en': 'British carrier', 'sv': u('Brittisk operat\u00f6r')},
 '44760':{'en': 'British pager'},
}



"""Auto-generated file, do not edit by hand. BR metadata"""
from phonenumbers.phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_BR = PhoneMetadata(id='BR', country_code=55, international_prefix='00',
    general_desc=PhoneNumberDesc(national_number_pattern='\\d{8,10}', possible_length=(10,), possible_length_local_only=(8,)),
    fixed_line=PhoneNumberDesc(national_number_pattern='\\d{8,10}', example_number='12345678', possible_length=(10,), possible_length_local_only=(8,)))



import unittest

from phonenumbers import PhoneMetadata
from phonenumbers import phonenumberutil

# Override library metadata with the test metadata.  First save the original
# data.
REAL_REGION_LOADERS = PhoneMetadata._region_available
REAL_COUNTRY_CODE_LOADERS = PhoneMetadata._country_code_available
REAL_REGION_METADATA = PhoneMetadata._region_metadata
REAL_COUNTRY_CODE_METADATA = PhoneMetadata._country_code_metadata
REAL_CC_TO_RC = phonenumberutil.COUNTRY_CODE_TO_REGION_CODE

# Clear the data dicts.
PhoneMetadata._region_available = {}
PhoneMetadata._country_code_available = {}
PhoneMetadata._region_metadata = {}
PhoneMetadata._country_code_metadata = {}
phonenumberutil.COUNTRY_CODE_TO_REGION_CODE = {}

# Import the test data; this will re-populate the cleared
# PhoneMetadata._region_available and PhoneMetadata._country_code_available maps
from .testdata import _COUNTRY_CODE_TO_REGION_CODE as TEST_CC_TO_RC
TEST_REGION_LOADERS = PhoneMetadata._region_available
TEST_COUNTRY_CODE_LOADERS = PhoneMetadata._country_code_available
TEST_REGION_METADATA = PhoneMetadata._region_metadata
TEST_COUNTRY_CODE_METADATA = PhoneMetadata._country_code_metadata


def reinstate_real_metadata():
    """Reinstate real phone number metadata"""
    phonenumberutil.COUNTRY_CODE_TO_REGION_CODE = REAL_CC_TO_RC
    PhoneMetadata._region_available = REAL_REGION_LOADERS
    PhoneMetadata._country_code_available = REAL_COUNTRY_CODE_LOADERS
    PhoneMetadata._region_metadata = REAL_REGION_METADATA
    PhoneMetadata._country_code_metadata = REAL_COUNTRY_CODE_METADATA
    phonenumberutil._regenerate_derived_data()


def insert_test_metadata():
    """Insert test metadata into library"""
    phonenumberutil.COUNTRY_CODE_TO_REGION_CODE = TEST_CC_TO_RC
    PhoneMetadata._region_available = TEST_REGION_LOADERS
    PhoneMetadata._country_code_available = TEST_COUNTRY_CODE_LOADERS
    PhoneMetadata._region_metadata = TEST_REGION_METADATA
    PhoneMetadata._country_code_metadata = TEST_COUNTRY_CODE_METADATA
    phonenumberutil._regenerate_derived_data()


# Reinstate the real metadata so any importers of this module are not affected
reinstate_real_metadata()


class TestMetadataTestCase(unittest.TestCase):
    def setUp(self):
        insert_test_metadata()

    def tearDown(self):
        reinstate_real_metadata()



"""Auto-generated file, do not edit by hand. BB metadata"""
from phonenumbers.phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_BB = PhoneMetadata(id='BB', country_code=1, international_prefix='011',
    general_desc=PhoneNumberDesc(national_number_pattern='246\\d{7}', possible_length=(10,), possible_length_local_only=(7,)),
    fixed_line=PhoneNumberDesc(example_number='2464567890'))



"""Auto-generated file, do not edit by hand. BS metadata"""
from phonenumbers.phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_BS = PhoneMetadata(id='BS', country_code=1, international_prefix='011',
    general_desc=PhoneNumberDesc(national_number_pattern='(242|8(00|66|77|88)|900)\\d{7}', possible_length=(10,), possible_length_local_only=(7,)),
    fixed_line=PhoneNumberDesc(national_number_pattern='242(?:3(?:02|[236][1-9]|4[0-24-9]|5[0-68]|7[3-57]|9[2-5])|4(?:2[237]|51|64|77)|502|636|702)\\d{4}', example_number='2425027890', possible_length=(10,), possible_length_local_only=(7,)),
    mobile=PhoneNumberDesc(national_number_pattern='242(357|359|457|557)\\d{4}', example_number='2423577890', possible_length=(10,)),
    toll_free=PhoneNumberDesc(national_number_pattern='8(00|66|77|88)\\d{7}', example_number='8001234567', possible_length=(10,)),
    premium_rate=PhoneNumberDesc(national_number_pattern='900\\d{7}', example_number='9001234567', possible_length=(10,)),
    national_prefix='1',
    national_prefix_for_parsing='1')



"""Unit tests for phonenumberutil.py"""

# Based on original Java code:
#     java/test/com/google/i18n/phonenumbers/ExampleNumbersTest.java
#
# Copyright (C) 2009 The Libphonenumber Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import sys
import unittest

from phonenumbers import PhoneNumberType, PhoneMetadata, NumberParseException
from phonenumbers import phonenumberutil, PhoneNumber, is_emergency_number
from phonenumbers import shortnumberinfo, ShortNumberCost, AsYouTypeFormatter
from phonenumbers import is_possible_short_number_for_region
from phonenumbers.util import prnt


class ExampleNumbersTest(unittest.TestCase):
    """Verifies all of the example numbers in the metadata are valid and of
    the correct type. If no example number exists for a particular type, the
    test still passes  since not all types are relevant for all regions.
    Tests that check the XML schema will ensure that an exampleNumber
    node is present for every phone number description.
    """

    def setUp(self):
        self.invalid_cases = []
        self.wrong_type_cases = []

    def tearDown(self):
        pass

    def _checkNumbersValidAndCorrectType(self,
                                         exampleNumberRequestedType,
                                         possibleExpectedTypes):
        """
        Arguments:
        exampleNumberRequestedType -- type we are requesting an example number for
        possibleExpectedTypes -- acceptable types that this number should match, such as
              FIXED_LINE and FIXED_LINE_OR_MOBILE for a fixed line example number.
        """
        for regionCode in phonenumberutil.SUPPORTED_REGIONS:
            exampleNumber = phonenumberutil.example_number_for_type(regionCode, exampleNumberRequestedType)
            if exampleNumber is not None:
                if not phonenumberutil.is_valid_number(exampleNumber):
                    self.invalid_cases.append(exampleNumber)
                    prnt("Failed validation for %s" % exampleNumber, file=sys.stderr)
                else:
                    # We know the number is valid, now we check the type.
                    exampleNumberType = phonenumberutil.number_type(exampleNumber)
                    if exampleNumberType not in possibleExpectedTypes:
                        self.wrong_type_cases.append(exampleNumber)
                        prnt("Wrong type for %s: got %s" % (exampleNumber, exampleNumberType), file=sys.stderr)
                        prnt("Expected types: ", file=sys.stderr)
                        for phone_type in possibleExpectedTypes:
                            prnt("  %s" % phone_type, file=sys.stderr)

    def testFixedLine(self):
        fixedLineTypes = set((PhoneNumberType.FIXED_LINE, PhoneNumberType.FIXED_LINE_OR_MOBILE))
        self._checkNumbersValidAndCorrectType(PhoneNumberType.FIXED_LINE, fixedLineTypes)
        self.assertEqual(0, len(self.invalid_cases))
        self.assertEqual(0, len(self.wrong_type_cases))

    def testMobile(self):
        mobileTypes = set((PhoneNumberType.MOBILE, PhoneNumberType.FIXED_LINE_OR_MOBILE,))
        self._checkNumbersValidAndCorrectType(PhoneNumberType.MOBILE, mobileTypes)
        self.assertEqual(0, len(self.invalid_cases))
        self.assertEqual(0, len(self.wrong_type_cases))

    def testTollFree(self):
        tollFreeTypes = set((PhoneNumberType.TOLL_FREE,))
        self._checkNumbersValidAndCorrectType(PhoneNumberType.TOLL_FREE, tollFreeTypes)
        self.assertEqual(0, len(self.invalid_cases))
        self.assertEqual(0, len(self.wrong_type_cases))

    def testPremiumRate(self):
        premiumRateTypes = set((PhoneNumberType.PREMIUM_RATE,))
        self._checkNumbersValidAndCorrectType(PhoneNumberType.PREMIUM_RATE, premiumRateTypes)
        self.assertEqual(0, len(self.invalid_cases))
        self.assertEqual(0, len(self.wrong_type_cases))

    def testVoip(self):
        voipTypes = set((PhoneNumberType.VOIP,))
        self._checkNumbersValidAndCorrectType(PhoneNumberType.VOIP, voipTypes)
        self.assertEqual(0, len(self.invalid_cases))
        self.assertEqual(0, len(self.wrong_type_cases))

    def testPager(self):
        pagerTypes = set((PhoneNumberType.PAGER,))
        self._checkNumbersValidAndCorrectType(PhoneNumberType.PAGER, pagerTypes)
        self.assertEqual(0, len(self.invalid_cases))
        self.assertEqual(0, len(self.wrong_type_cases))

    def testUan(self):
        uanTypes = set((PhoneNumberType.UAN,))
        self._checkNumbersValidAndCorrectType(PhoneNumberType.UAN, uanTypes)
        self.assertEqual(0, len(self.invalid_cases))
        self.assertEqual(0, len(self.wrong_type_cases))

    def testVoicemail(self):
        voicemailTypes = set((PhoneNumberType.VOICEMAIL,))
        self._checkNumbersValidAndCorrectType(PhoneNumberType.VOICEMAIL, voicemailTypes)
        self.assertEqual(0, len(self.invalid_cases))
        self.assertEqual(0, len(self.wrong_type_cases))

    def testSharedCost(self):
        sharedCostTypes = set((PhoneNumberType.SHARED_COST,))
        self._checkNumbersValidAndCorrectType(PhoneNumberType.SHARED_COST, sharedCostTypes)
        self.assertEqual(0, len(self.invalid_cases))
        self.assertEqual(0, len(self.wrong_type_cases))

    def testCanBeInternationallyDialled(self):
        for regionCode in phonenumberutil.SUPPORTED_REGIONS:
            exampleNumber = None
            metadata = PhoneMetadata.metadata_for_region(regionCode, None)
            desc = None
            if metadata is not None:
                desc = metadata.no_international_dialling
            try:
                if desc is not None and desc.example_number is not None:
                    exampleNumber = phonenumberutil.parse(desc.example_number, regionCode)

            except NumberParseException:
                _, e, _ = sys.exc_info()
                prnt("Failed parse: %s" % e, file=sys.stderr)

            if (exampleNumber is not None and
                phonenumberutil.can_be_internationally_dialled(exampleNumber)):
                self.wrong_type_cases.append(exampleNumber)
                prnt("Number %s should not be internationally diallable" % exampleNumber, file=sys.stderr)
        self.assertEqual(0, len(self.wrong_type_cases))

    def testGlobalNetworkNumbers(self):
        PhoneMetadata.load_all()
        for callingCode in PhoneMetadata._country_code_metadata.keys():
            exampleNumber = phonenumberutil.example_number_for_non_geo_entity(callingCode)
            self.assertTrue(exampleNumber is not None,
                            msg="No example phone number for calling code %s" % callingCode)
            if not phonenumberutil.is_valid_number(exampleNumber):
                self.invalid_cases.append(exampleNumber)
                prnt("Failed validation for %s" % exampleNumber, file=sys.stderr)
        self.assertEqual(0, len(self.invalid_cases))

    def testEveryRegionHasAnExampleNumber(self):
        for regionCode in phonenumberutil.SUPPORTED_REGIONS:
            exampleNumber = phonenumberutil.example_number(regionCode)
            self.assertTrue(exampleNumber is not None,
                            msg="No example number found for region %s" % regionCode)

    def testEveryRegionHasAnInvalidExampleNumber(self):
        for regionCode in phonenumberutil.SUPPORTED_REGIONS:
            exampleNumber = phonenumberutil.invalid_example_number(regionCode)
            self.assertTrue(exampleNumber is not None,
                            msg="No invalid example number found for region %s" % regionCode)

    def testEveryTypeHasAnExampleNumber(self):
        for num_type in PhoneNumberType.values():
            if num_type == PhoneNumberType.UNKNOWN:
                continue
            exampleNumber = phonenumberutil.example_number_for_type(None, num_type)
            self.assertTrue(exampleNumber is not None,
                            msg="No example number found for type %s" % num_type)

    def testShortNumbersValidAndCorrectCost(self):
        invalid_string_cases = []
        for regionCode in shortnumberinfo.SUPPORTED_SHORT_REGIONS:
            exampleShortNumber = shortnumberinfo._example_short_number(regionCode)
            phoneNumber = phonenumberutil.parse(exampleShortNumber, regionCode)
            if not shortnumberinfo.is_valid_short_number_for_region(phoneNumber, regionCode):
                invalid_string_case = "region_code: %s, national_number: %s" % (regionCode, exampleShortNumber)
                invalid_string_cases.append(invalid_string_case)
                prnt("Failed validation from string %s" % invalid_string_case, file=sys.stderr)
            if not shortnumberinfo.is_valid_short_number(phoneNumber):
                self.invalid_cases.append(phoneNumber)
                prnt("Failed validation for %s" % phoneNumber, file=sys.stderr)
            for cost in [ShortNumberCost.TOLL_FREE, ShortNumberCost.STANDARD_RATE,
                         ShortNumberCost.PREMIUM_RATE, ShortNumberCost.UNKNOWN_COST]:
                exampleShortNumber = shortnumberinfo._example_short_number_for_cost(regionCode, cost)
                if exampleShortNumber != "":
                    phoneNumber = phonenumberutil.parse(exampleShortNumber, regionCode)
                    exampleCost = shortnumberinfo.expected_cost_for_region(phoneNumber, regionCode)
                    if cost != exampleCost:
                        self.wrong_type_cases.append(phoneNumber)
                        prnt("Wrong cost for %s: got %s, expected: %s" %
                             (phoneNumber, exampleCost, cost), file=sys.stderr)
        self.assertEqual(0, len(invalid_string_cases))
        self.assertEqual(0, len(self.invalid_cases))
        self.assertEqual(0, len(self.wrong_type_cases))

    def testEmergency(self):
        wrongTypeCounter = 0
        for regionCode in shortnumberinfo.SUPPORTED_SHORT_REGIONS:
            metadata = PhoneMetadata.short_metadata_for_region(regionCode, None)
            desc = metadata.emergency
            if desc is not None and desc.example_number is not None:
                exampleNumber = desc.example_number
                phoneNumber = phonenumberutil.parse(exampleNumber, regionCode)
                if (not is_possible_short_number_for_region(phoneNumber, regionCode) or
                    not is_emergency_number(exampleNumber, regionCode)):
                    wrongTypeCounter += 1
                    prnt("Emergency example number test failed for %s" % regionCode, file=sys.stderr)
                elif shortnumberinfo.expected_cost_for_region(phoneNumber, regionCode) != ShortNumberCost.TOLL_FREE:
                    wrongTypeCounter += 1
                    prnt("Emergency example number not toll free for %s" % regionCode, file=sys.stderr)
        self.assertEqual(0, wrongTypeCounter)

    def testCarrierSpecificShortNumbers(self):
        wrongTagCounter = 0
        for regionCode in shortnumberinfo.SUPPORTED_SHORT_REGIONS:
            metadata = PhoneMetadata.short_metadata_for_region(regionCode, None)
            desc = metadata.carrier_specific
            if desc is not None and desc.example_number is not None:
                exampleNumber = desc.example_number
                carrierSpecificNumber = phonenumberutil.parse(exampleNumber, regionCode)
                if (not shortnumberinfo.is_possible_short_number_for_region(carrierSpecificNumber, regionCode) or
                    not shortnumberinfo.is_carrier_specific_for_region(carrierSpecificNumber, regionCode)):
                    wrongTagCounter += 1
                    prnt("Carrier-specific test failed for %s" % regionCode, file=sys.stderr)
        self.assertEqual(0, wrongTagCounter)

    def testSmsServiceShortNumbers(self):
        wrongTagCounter = 0
        for regionCode in shortnumberinfo.SUPPORTED_SHORT_REGIONS:
            metadata = PhoneMetadata.short_metadata_for_region(regionCode, None)
            desc = metadata.sms_services
            if desc is not None and desc.example_number is not None:
                exampleNumber = desc.example_number
                smsServiceNumber = phonenumberutil.parse(exampleNumber, regionCode)
                if (not shortnumberinfo.is_possible_short_number_for_region(smsServiceNumber, regionCode) or
                    not shortnumberinfo.is_sms_service_for_region(smsServiceNumber, regionCode)):
                    wrongTagCounter += 1
                    prnt("SMS service test failed for %s" % regionCode, file=sys.stderr)
        self.assertEqual(0, wrongTagCounter)

    def testIsCarrierSpecific(self):
        # Python version extra test: hit is_carrier_specific entrypoint
        esNumber = PhoneNumber(country_code=34, national_number=123)
        self.assertTrue(shortnumberinfo.is_carrier_specific(esNumber))
        esNumber.national_number = 512345678
        self.assertFalse(shortnumberinfo.is_carrier_specific(esNumber))

    # Extra tests that need access to the real metadata
    def testMissingShortMetadata(self):
        # Python version extra test:
        # Check a short number for a country (GQ) that has no short metadata
        numobj = PhoneNumber(country_code=240, national_number=234123456)
        self.assertFalse(shortnumberinfo.is_possible_short_number(numobj))

    def testBlankMetadata(self):
        # Python version extra test
        # Some metadata is blank; check that we cope with this.
        # Example: MH (+692)
        number = phonenumberutil.parse("+6927654321", "US")
        self.assertEqual("Country Code: 692 National Number: 7654321", str(number))

    def testMetadataPrint(self):
        # Python version extra test
        # Convert all metadata to strings to check the printing code doesn't blow up.
        for region_code in PhoneMetadata._region_available.keys():
            metadata = PhoneMetadata.metadata_for_region(region_code)
            str(metadata)

    def testWhitespaceInNationalPrefixForParsing(self):
        # Python version extra test
        # AR metadata has whitespace in the RE for nationalPrefixForParsing
        number = phonenumberutil.parse("+540348215617137", "AR")
        self.assertTrue(phonenumberutil.is_valid_number(number))
        self.assertEqual(PhoneNumberType.MOBILE, phonenumberutil.number_type(number))
        number = phonenumberutil.parse("0344615614207", "AR")
        self.assertTrue(phonenumberutil.is_valid_number(number))
        self.assertEqual(PhoneNumberType.MOBILE, phonenumberutil.number_type(number))

    def testFormatNumberForMobile(self):
        # Python version extra test.  Special cases for CO and BR in
        # format_number_for_mobile_dialing(), included here so that real metadata is used
        coNumberFixed = PhoneNumber(country_code=57, national_number=12345678)
        coNumberMobile = PhoneNumber(country_code=57, national_number=3211234567)
        peNumberFixed = PhoneNumber(country_code=51, national_number=11234567)
        brNumberFixed = PhoneNumber(country_code=55, national_number=1123456789)
        brNumberMobile = PhoneNumber(country_code=55, national_number=11961234567,
                                     preferred_domestic_carrier_code="303")
        huNumberFixed = PhoneNumber(country_code=36, national_number=12345678)
        self.assertEqual("0312345678",
                         phonenumberutil.format_number_for_mobile_dialing(coNumberFixed, "CO", False))
        self.assertEqual("03 1 2345678",
                         phonenumberutil.format_number_for_mobile_dialing(coNumberFixed, "CO", True))
        self.assertEqual("3211234567",
                         phonenumberutil.format_number_for_mobile_dialing(coNumberMobile, "CO", False))
        self.assertEqual("321 1234567",
                         phonenumberutil.format_number_for_mobile_dialing(coNumberMobile, "CO", True))
        self.assertEqual("011234567",
                         phonenumberutil.format_number_for_mobile_dialing(peNumberFixed, "PE", False))
        self.assertEqual("(01) 1234567",
                         phonenumberutil.format_number_for_mobile_dialing(peNumberFixed, "PE", True))
        self.assertEqual("",
                         phonenumberutil.format_number_for_mobile_dialing(brNumberFixed, "BR", False))
        self.assertEqual("",
                         phonenumberutil.format_number_for_mobile_dialing(brNumberFixed, "BR", True))
        self.assertEqual("030311961234567",
                         phonenumberutil.format_number_for_mobile_dialing(brNumberMobile, "BR", False))
        self.assertEqual("0 303 (11) 96123-4567",
                         phonenumberutil.format_number_for_mobile_dialing(brNumberMobile, "BR", True))
        self.assertEqual("0612345678",
                         phonenumberutil.format_number_for_mobile_dialing(huNumberFixed, "HU", False))

    def testAYTFShortNumberFormatting_AR(self):
        # Python version extra test: use real metadata so that the check for accrued digits already
        # matching a format fires.
        formatter = AsYouTypeFormatter("AR")
        self.assertEqual("1", formatter.input_digit('1'))
        self.assertEqual("10", formatter.input_digit('0'))
        self.assertEqual("101", formatter.input_digit('1'))

    def testPrintShortMetadata(self):
        # Python version extra test.  Print string representation of short metadata.
        short_metadata = PhoneMetadata.short_metadata_for_region("GB")
        result = str(short_metadata)
        self.assertTrue(result.startswith("PhoneMetadata(id='GB', country_code=None, international_prefix=None"))

    def testGBLocalNumberLength(self):
        # Python version extra test.  Issue #172.
        numobj = phonenumberutil.parse("+4408001111", "GB")
        self.assertEqual("+44 800 1111", phonenumberutil.format_number(numobj, phonenumberutil.PhoneNumberFormat.INTERNATIONAL))
